#include <linux/soundcard.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdint.h>

extern char *dev;
extern int midifd;

#define NOTEOFF        8
#define NOTEON         9
#define AFTERTOUCH_P  10
#define CONTROLCHANGE 11
#define PROGCHANGE    12
#define AFTERTOUCH_C  13
#define PITCHBEND     14

#define CHANNELS     16
#define CHANNEL_NONE CHANNELS

static inline int cmd_len(uint8_t command) {
  switch (command) {
    case NOTEOFF:      return 3;
    case NOTEON:       return 3;
    case AFTERTOUCH_P: return 3;
    case AFTERTOUCH_C: return 2;
    case PITCHBEND:    return 3;
    case CONTROLCHANGE: return 3;
    case PROGCHANGE:   return 2;
    default:
      printf("unexpected command %d\n", command);
      return 0;    
  }
}

static inline void cmd(uint8_t channel, uint8_t command, uint8_t note, uint8_t velocity) {
  uint8_t data[3] = {channel | (command << 4), note, velocity};
  write(midifd, data, cmd_len(command));
}

#define note_on(chan, note, vel) cmd(chan, NOTEON, note, vel)
#define note_off(chan, note, vel) cmd(chan, NOTEOFF, note, vel)
#define pitchbend(chan, msb, lsb) cmd(chan, PITCHBEND, lsb, msb)
#define aftertouch(chan, note, vel) cmd(chan, AFTERTOUCH_P, note, vel)
#define allnoteoff(chan) cmd(chan, CONTROLCHANGE, 120, 0)

#define bankmsbselect(chan, bank) cmd(chan, CONTROLCHANGE, 0, bank)
#define banklsbselect(chan, bank) cmd(chan, CONTROLCHANGE, 32, bank)
#define patchselect(chan, patch) cmd(chan, PROGCHANGE, patch, 0)

static inline void instrument(uint8_t chan, uint8_t msb, uint8_t lsb, uint8_t patch) {
  bankmsbselect(chan, msb);
  banklsbselect(chan, lsb);
  patchselect(chan, patch);
}

#define chanvol(chan, volume) cmd(chan, CONTROLCHANGE, 7, volume)
#define pan(chan, val) cmd(chan, CONTROLCHANGE, 10, val)
